### -*-awk-*-
### ====================================================================
###  @Awk-file{
###     author          = "Nelson H. F. Beebe",
###     version         = "0.11",
###     date            = "13 January 2000",
###     time            = "11:23:19 MST",
###     filename        = "psposter.sh",
###     copyright       = "Copyright (C) 2000 Nelson H. F. Beebe",
###     address         = "Center for Scientific Computing
###                        University of Utah
###                        Department of Mathematics, 322 INSCC
###                        155 S 1400 E RM 233
###                        Salt Lake City, UT 84112-0090
###                        USA",
###     telephone       = "+1 801 581 5254",
###     FAX             = "+1 801 585 1640, +1 801 581 4148",
###     URL             = "http://www.math.utah.edu/~beebe",
###     checksum        = "63595 1608 7211 56572",
###     email           = "beebe@math.utah.edu, beebe@acm.org,
###                        beebe@ieee.org (Internet)",
###     codetable       = "ISO/ASCII",
###     keywords        = "poster, PostScript",
###     supported       = "yes",
###     docstring       = "Display a centered message in a very large
###                        font for a poster, which is represented as an
###                        output Encapsulated PostScript file which
###                        will produce exactly one page.
###
###                        Usage:
###                             psposter [+author] [+bd color] [+bg color]
###					 [+bm dimension] [+bs factor]
###					 [+bw dimension] [+c nnn]
###					 [+copyright] [+d ] [+e encoding]
###					 [+fg color] [+fn fontname]
###					 [+h dimension] [+help] [+l]
###					 [+mw dimension] [+p dimension]
###					 [+paper form] [+s factor] [+source]
###					 [+table {8/o | 8/h | 16/o | 16/h}]
###					 [+version] [+w dimension]
###					 [+x dimension] [+y dimension]
###					 ['?'] [++]
###                                      [word(s) or <infile]
###                                      >Encapsulated-PostScript-file
###
###                        Complete documentation of the command-line
###                        options can be found in the accompanying UNIX
###                        manual pages.
###
###                        The font size is normally chosen
###                        automatically so that the longest line fills
###                        the page, but a fixed font size may be
###                        requested if required.
###
###                        If there are too many lines to fit on one
###                        page, lines will be lost at the top and
###                        bottom of the page.
###
###                        Options are provided to change colors, to
###                        request multiple copies, to adjust the
###                        position, scale, and orientation, and to
###                        select the font and character-set encoding.
###
###                        Each poster line consists of one argument,
###                        if the words come from the command line, or
###                        one line, if the words come from a file.
###
###                        The checksum field above contains a CRC-16
###                        checksum as the first value, followed by the
###                        equivalent of the standard UNIX wc (word
###                        count) utility output of lines, words, and
###                        characters.  This is produced by Robert
###                        Solovay's checksum utility.",
###  }
### ====================================================================

BEGIN {
    Stderr = "/dev/stderr"	# or "/dev/tty" if "/dev/stderr" unknown
    VERSION_NUMBER = "0.11"
    VERSION = "Version " VERSION_NUMBER " [13-Jan-2000]"
    AFMPATH = ENVIRON["AFMPATH"]
    if (AFMPATH == "")			# supply default fallback search path
	AFMPATH = "/usr/local/share/lib/afm"
    split(AFMPATH,AFM_path,":")
    AFMMAPFILE = find_file(AFM_path,"afmfonts.map")
    if (AFMMAPFILE == "")
	warning("Cannot find font mapping file [afmfonts.map]")

    DEFAULT_FONT_NAME = "Times-Bold"

    RGBPATH = ENVIRON["RGBPATH"]
    if (RGBPATH == "")			# supply default fallback search path
	RGBPATH = "/usr/lib/X11:/usr/openwin/lib/X11:/usr/local/lib/X11:/usr/local/X11R5/lib:/usr/lpp/X11/lib/X11"
    split(RGBPATH,rgbpath,":")
    RGBTXTFILE = find_file(rgbpath,"rgb.txt")
    if (RGBTXTFILE == "")
	warning("Cannot find X11 color mapping file [rgb.txt]")

    Angle = ""				# empty string means unset
    Background_Color = "0.9 0.9 0.9"		# 10% gray
    Baseline_Skip = 1.2
    Border_Color = "0.5 0.5 0.5"	# 50% gray
    Border_Margin = 0			# big points
    Border_Width = 10			# big points
    Copies = 1
    Display_Fontsize = 0
    Embedded_Font_Count = 0
    Font_Name = DEFAULT_FONT_NAME
    Foreground_Color = "0 0 0"		# black
    Height = 0
    Width = 0
    Landscape = 0
    Make_Font_Table = 0
    Margin_Width = ""			# empty string means unset
    Paper_Name = ""
    Point_Size = 0
    ScaleFactor = 1
    X_Offset = 0
    Y_Offset = 0

    initialize_paper_table()

    for (k = 1; k < ARGC; ++k) # preserve original command-line arguments
	OLD_ARGV[k] = ARGV[k]

    ## for (k = 1; k < ARGC; k++) print "DEBUG: PRE: ARGV[" k "] = [" ARGV[k] "]" >"/dev/tty"

    NWord = 0			# index into Word[]
    for (k = 1; k < ARGC; k++)	# collect +switches
    {
	if (ARGV[k] == "+author")
	    author()
	else if (ARGV[k] == "+bd")
	{
	    ARGV[k] = ""	# forget this argument
	    Border_Color = ARGV[++k]
	    ARGV[k] = ""	# forget this argument
	}
	else if (ARGV[k] == "+bg")
	{
	    ARGV[k] = ""	# forget this argument
	    Background_Color = ARGV[++k]
	    ARGV[k] = ""	# forget this argument
	}
	else if (ARGV[k] == "+bm")
	{
	    ARGV[k] = ""	# forget this argument
	    Border_Margin = bigpoint(ARGV[++k])
	    ARGV[k] = ""	# forget this argument
	}
	else if (ARGV[k] == "+bs")
	{
	    ARGV[k] = ""	# forget this argument
	    Baseline_Skip = ARGV[++k] + 0
	    ARGV[k] = ""	# forget this argument
	}
	else if (ARGV[k] == "+bw")
	{
	    ARGV[k] = ""	# forget this argument
	    Border_Width = bigpoint(ARGV[++k])
	    ARGV[k] = ""	# forget this argument
	}
	else if (ARGV[k] == "+c")
	{
	    ARGV[k] = ""	# forget this argument
	    Copies = ARGV[++k] + 0
	    ARGV[k] = ""	# forget this argument
	}
	else if (ARGV[k] == "+copyright")
	    copyright()
	else if (ARGV[k] == "+d")
	{
	    ARGV[k] = ""	# forget this argument
	    Display_Fontsize = 1
	}
	else if (ARGV[k] == "+e")
	{
	    ARGV[k] = ""	# forget this argument
	    Encoding = ARGV[++k]
	    ARGV[k] = ""	# forget this argument
	}
	else if (ARGV[k] == "+fg")
	{
	    ARGV[k] = ""	# forget this argument
	    Foreground_Color = ARGV[++k]
	    ARGV[k] = ""	# forget this argument
	}
	else if (ARGV[k] == "+fn")
	{
	    ARGV[k] = ""	# forget this argument
	    Font_Name = ARGV[++k]
	    ARGV[k] = ""	# forget this argument
	}
	else if (ARGV[k] == "+h")
	{
	    ARGV[k] = ""	# forget this argument
	    Height = bigpoint(ARGV[++k])
	    ARGV[k] = ""	# forget this argument
	}
	else if ((ARGV[k] == "+help") || (ARGV[k] == "?"))
	    usage()
	else if (ARGV[k] == "+l")
	{
	    ARGV[k] = ""	# forget this argument
	    Landscape = 1
	    Angle = +90
	}
	else if (ARGV[k] == "+mw")
	{
	    ARGV[k] = ""	# forget this argument
	    Margin_Width = bigpoint(ARGV[++k])
	    ARGV[k] = ""	# forget this argument
	}
	else if (ARGV[k] == "+s")
	{
	    ARGV[k] = ""	# forget this argument
	    ScaleFactor = ARGV[++k] + 0
	    ARGV[k] = ""	# forget this argument
	}
	else if (ARGV[k] == "+source")
	    source()
	else if (ARGV[k] == "+table")
	{
	    ARGV[k] = ""	# forget this argument
	    parse_table_type(ARGV[++k])
	    ARGV[k] = ""	# forget this argument
	}
	else if (ARGV[k] == "+p")
	{
	    ARGV[k] = ""	# forget this argument
	    Point_Size = bigpoint(ARGV[++k])
	    ARGV[k] = ""	# forget this argument
	}
	else if (ARGV[k] == "+paper")
	{
	    ARGV[k] = ""	# forget this argument
	    set_paper_dimensions(ARGV[++k])
	    ARGV[k] = ""	# forget this argument
	}
	else if (ARGV[k] == "+r")
	{
	    ARGV[k] = ""	# forget this argument
	    Angle = ARGV[++k] + 0
	    ARGV[k] = ""	# forget this argument
	    Angle = Angle % 360	# reduce to -360 .. 360
	    if (Angle < 0)	# and reduce again to 0 .. 359
		Angle += 360
	    if (!((Angle == 0) || (Angle == 90) || (Angle == 180) || (Angle == 270)))
		error("rotation angle is not a multiple of 90 degrees in -360 .. +360")
	}
	else if (ARGV[k] == "+version")
	    version()
	else if (ARGV[k] == "+w")
	{
	    ARGV[k] = ""	# forget this argument
	    Width = bigpoint(ARGV[++k])
	    ARGV[k] = ""	# forget this argument
	}
	else if (ARGV[k] == "+x")
	{
	    ARGV[k] = ""	# forget this argument
	    X_Offset = bigpoint(ARGV[++k])
	    ARGV[k] = ""	# forget this argument
	}
	else if (ARGV[k] == "+y")
	{
	    ARGV[k] = ""	# forget this argument
	    Y_Offset = bigpoint(ARGV[++k])
	    ARGV[k] = ""	# forget this argument
	}
	else if (ARGV[k] == "++") # no more arguments
	{
	    ARGV[k] = ""	# forget this argument
	    while (k < ARGC)	# remaining words are for poster text
	    {
		Word[++NWord] = ARGV[++k]
		ARGV[k] = ""	# forget this argument
	    }
	}
	else if (ARGV[k] ~ /^[ \t]*[+]/)
	    error("unrecognized option [" ARGV[k] "]: try +help for more information")
	else
	    Word[++NWord] = ARGV[k]
    }

    ## for (k = 1; k < ARGC; k++) print "DEBUG: POST: ARGV[" k "] = [" ARGV[k] "]" >"/dev/tty"

    if ((NWord == 0) && (Make_Font_Table == 0))
    {			# no words supplied, so read stdin for lines to print
	while (getline < "-" > 0)
	    Word[++NWord] = $0
    }

    if (Paper_Name == "")
	set_paper_dimensions("A")

    if (Margin_Width == "")
	Margin_Width = (Paper_Left + Paper_Right + Paper_Top + Paper_Bottom)/4
    if (Height == 0)
	Height = Paper_Height
    if (Width == 0)
	Width = Paper_Width

    ##
    ##   Here is what the picture will look like.  P's mark the PAGE
    ##   boundary, enclosing a region of size (PageWidth,PageHeight).
    ##   M's mark the MARGIN, B's mark the border, and m's mark the
    ##   border margin; all three have constant width on all four
    ##   edges.  The inner frame contains the poster text; it has width
    ##   W (TextWidth) and height H (TextHeight).  The lower-left
    ##   corner of the MARGIN area is at (xoffset,yoffset) from the
    ##   lower-left page corner.  The BoundingBox surrounds the border
    ##   rectangle marked by Bs.
    ##
    ##   For full page posters, normally (xoffset,yoffset) = (0,0),
    ##   but the MarginWidth is non-zero, because most PostScript
    ##   output devices are incapable of printing closer than about
    ##   5mm to the edge of the printing surface.
    ##
    ##   The inner frame may be further scaled by ScaleFactor;
    ##   that scaling applies to everything, EXCEPT
    ##   (xoffset,yoffset), because those offsets are used to
    ##   position the scaled picture on the page.
    ##
    ##           PPPPPPPPPPPPPPPPPPPPPPPP
    ##           P                      P
    ##           P                      P
    ##           P  MMMMMMMMMMMMMMMMMM  P
    ##           P  MMMMMMMMMMMMMMMMMM  P
    ##           P  MMBBBBBBBBBBBBBBMM  P
    ##           P  MMBmmmmmmmmmmmmBMM  P
    ##           P  MMBm ^        mBMM  P
    ##           P  MMBm |        mBMM  P
    ##           P  MMBm |        mBMM  P
    ##           P  MMBm<+----W-->mBMM  P
    ##           P  MMBm |        mBMM  P
    ##           P  MMBm H        mBMM  P
    ##           P  MMBm |        mBMM  P
    ##           P  MMBm |        mBMM  P
    ##           P  MMBm |        mBMM  P
    ##           P  MMBm v        mBMM  P
    ##           P  MMBmmmmmmmmmmmmBMM  P
    ##           P  MMBBBBBBBBBBBBBBMM  P
    ##           P  MMMMMMMMMMMMMMMMMM  P
    ##           P  MMMMMMMMMMMMMMMMMM  P
    ##         ^ P                      P
    ##         | P                      P
    ## yoffset | P                      P
    ##         | P                      P
    ##         v PPPPPPPPPPPPPPPPPPPPPPPP
    ##           <->xoffset
    ##

    ## NB: Text_Width and Text_Height must match the definitions of the
    ## PostScript values TextWidth and TextHeight below
    set_text_dimensions()

    if (Text_Height < Point_Size)
    {
	warning("tight fit: assuming +bm 0")
	Border_Margin = 0
	set_text_dimensions()
	if (Text_Height < Point_Size)
	{
	    warning("tight fit: assuming +mw 0")
	    Margin_Width = 0
	    set_text_dimensions()
	    if (Text_Height < Point_Size)
	    {
		warning("tight fit: assuming +bw 0")
		Border_Width = 0
		set_text_dimensions()
	    }
	}
    }
    if (Text_Height < Point_Size)
	error("Page, margin, or border dimensions too small for requested point size")

    if (Text_Width < Point_Size)
    {
	warning("tight fit: assuming +bm 0")
	Border_Margin = 0
	set_text_dimensions()
	if (Text_Width < Point_Size)
	{
	    warning("tight fit: assuming +mw 0")
	    Margin_Width = 0
	    set_text_dimensions()
	    if (Text_Width < Point_Size)
	    {
		warning("tight fit: assuming +bw 0")
		Border_Width = 0
		set_text_dimensions()
	    }
	}
    }
    if (Text_Width < Point_Size)
	error("Page, margin, or border dimensions too small for requested point size")

    if (dimen_err > 0)
	warning("One or more of +bm, +mw, or +bw values have been set to zero to squeeze text into small page")

    current_date_and_time = shell_command("date")

    if ("USER" in ENVIRON)
	username = ENVIRON["USER"]
    else if ("LOGNAME" in ENVIRON)
	username = ENVIRON["LOGNAME"]
    else
	username = "unknown"
    personal_name = shell_command("awk -F: '/^" username ":/ {print $5}' /etc/passwd")
    if (personal_name == "")
	personal_name = shell_command("ypcat passwd 2>/dev/null | awk -F: '/^" username ":/ {print $5}'")
    hostname = shell_command("hostname")

    ## Get the file name of any font file to be downloaded, since we
    ## need to know how to describe it in the header comments.
    font_filename = find_font(Font_Name)
    get_embedded_font_names(font_filename)

    print "%!PS-Adobe-3.0 EPSF-3.0"
    printf("%%%%Title: psposter")
    for (k = 1; k < ARGC; ++k)
	printf(" %s",OLD_ARGV[k])
    print ""

    print "%%CreationDate:", current_date_and_time
    print "%%Creator: psposter", VERSION
    print "%%For: " personal_name " <" username "@" hostname ">"
    llx = floor(X_Offset + ScaleFactor * Margin_Width)
    lly = floor(Y_Offset + ScaleFactor * Margin_Width)
    urx = ceil(X_Offset + \
	    ScaleFactor * (Margin_Width + Text_Width + 2*Border_Width + \
			   2*Border_Margin))
    ury = ceil(Y_Offset + \
	    ScaleFactor * (Margin_Width + Text_Height + 2*Border_Width + \
			   2*Border_Margin))
    if ((llx >= urx) || (lly >= ury))
	error("empty bounding box: margins and/or borders too big for page size")
    print "%%BoundingBox:", llx, llx, urx, ury

    print "%%DocumentMedia:", Paper_Name, \
	ceil(Paper_Width), ceil(Paper_Height), "0 () ()"
    print "%%DocumentData: Clean7Bit"

    font_list = ""
    for (k = Embedded_Font_Count ; k > 0; --k)
	font_list = font_list Embedded_Font_Names[k] " "
    font_list = font_list Font_Name
    print "%%DocumentFonts: font", font_list

    if (font_filename == "")		# printer-resident font
	print "%%DocumentNeededResources: font", Font_Name
    else				# downloaded font(s)
	print "%%DocumentSuppliedResources: font", font_list

    print "%%Orientation:", (Landscape ? "Landscape" : "Portrait")
    print "%%PageOrder: Ascend"
    print "%%Pages: 1"
    print "%%EndComments"
    print "%======================================================================="
    print "%%BeginProlog"
    print "/psposterdict 50 dict def"
    print "psposterdict begin"
    print "/inch { 72 mul } def"
    print "/BackgroundColor {", getcolor(Background_Color), "} def %",Background_Color
    print "/BaselineSkipFactor", Baseline_Skip, "def"
    print "/BorderColor {", getcolor(Border_Color), "} def %",Border_Color
    print "/BorderMargin", Border_Margin, "def"
    print "/BorderWidth", Border_Width, "def"
    print "/CapHeight"
    print "{"
    print "\tgsave newpath 0 0 moveto (M) true charpath pathbbox"
    print "\texch pop sub neg exch pop grestore"
    print "} def"
    print "/ForegroundColor {", getcolor(Foreground_Color), "} def %",Foreground_Color
    if (Angle != "")			# "+rotation angle" overrides "+l"
	Landscape = int(Angle / 90) % 2

    if (Encoding == "")
	print_default_encoding()
    else
	print_private_encoding()

    print "/reencodePRIVATE {"
    print "\tdup length dict begin"
    print "\t\t{1 index /FID ne {def}{pop pop} ifelse} forall"

    ## Special handling for StandardEncoding: we interpret this to mean
    ## the built-in encoding vector for the font, which we could
    ## retrieve by, e.g., /ZapfDingbats findfont /Encoding get.  If we
    ## were to reset Encoding to StandardEncoding, then fonts with
    ## unusual encodings, like Symbol and ZapfDingbats, would be
    ## unusable for posters.
    if (Encoding == "StandardEncoding")
	print "\t\t%% Using default font encoding, perhaps: /Encoding " Encoding " def"
    else
	print "\t\t/Encoding " Encoding " def"

    print "\t\tcurrentdict"
    print "\tend"
    print "} def"
    print "/findPRIVATE {"
    print "\tdup /FontType known {"
    print "\t\tdup /FontType get 3 ne {"
    print "\t\t\tdup /CharStrings known {"
    print "\t\t\t\tdup /CharStrings get /Thorn known {"
    print "\t\t\t\t\ttrue"
    print "\t\t\t\t}{ false } ifelse"
    print "\t\t\t}{ false } ifelse"
    print "\t\t}{ false } ifelse"
    print "\t}{ false } ifelse"
    print "} def"

    print "/LandscapeMode", (Landscape ? "true" : "false"), "def"
    print "/MarginWidth", Margin_Width, "def"
    print "/PageHeight", Paper_Height, "def"
    print "/PageWidth",  Paper_Width,  "def"
    print "/Height", Height, "def"
    print "/Width",  Width,  "def"
    print "/PointSize", Point_Size, "def"
    print "/ROTATE_0 {} def"
    print "/ROTATE_90 {Width 0 translate 90 rotate} bind def"
    print "/ROTATE_180 {Width Height translate 180 rotate} bind def"
    print "/ROTATE_270 {0 Height translate 270 rotate} bind def"
    print "/ScaleFactor", ScaleFactor, "def"

    print "LandscapeMode"
    print "{"
    print "\t/TextHeight Width MarginWidth 2 mul sub BorderWidth 2 mul sub BorderMargin 2 mul sub def"
    print "\t/TextWidth Height MarginWidth 2 mul sub BorderWidth 2 mul sub BorderMargin 2 mul sub def"
    print "}"
    print "{"
    print "\t/TextHeight Height MarginWidth 2 mul sub BorderWidth 2 mul sub BorderMargin 2 mul sub def"
    print "\t/TextWidth Width MarginWidth 2 mul sub BorderWidth 2 mul sub BorderMargin 2 mul sub def"
    print "}"
    print "ifelse"

    print "/XOffset", X_Offset, "def"
    print "/YOffset", Y_Offset, "def"

    print "% Center and show a string at the current point, leaving"
    print "% point at the center of the next line."
    print "% Usage: (string) cshow"
    print "/cshow"
    print "{"
    print "\tcurrentpoint /cpy exch def /cpx exch def"
    print "\tdup stringwidth pop 0.5 mul neg 0 rmoveto show"
    print "\tcpx cpy moveto"
    print "\tNewLine"
    print "} def"

    print "/max"
    print "{"
    print "\t/a exch def"
    print "\t/b exch def"
    print "\ta b gt { a } { b } ifelse"
    print "} def"

    print "/min"
    print "{"
    print "\t/a exch def"
    print "\t/b exch def"
    print "\ta b lt { a } { b } ifelse"
    print "} def"

    print "/TheFont /" Font_Name " def"
    print "/TheFontPRIVATE /" Font_Name "-PRIVATE def"
    print "TheFontPRIVATE TheFont findfont reencodePRIVATE definefont pop"
    print "% Set the font size so that longest PosterLines string fills the TextWidth"
    print "% Usage: setfontsize"
    print "/setfontsize"
    print "{"
    print "\tTheFont findfont"
    print "\t\tfindPRIVATE {reencodePRIVATE TheFontPRIVATE exch definefont} if"
    print "\t\t10 scalefont setfont"
    print "\t% Set TheFontSize according to maximum string width"
    print "\t0 PosterLines { stringwidth pop max } forall TextWidth exch div 10 mul"
    print "\t/TheFontSize exch def"
    print "\t% Adjust TheFontSize according to the number of text lines"
    print "\tLineCount TheFontSize mul BaselineSkipFactor mul"
    print "\tTextHeight div"
    print "\tdup 1 gt { TheFontSize exch div /TheFontSize exch def } { pop } ifelse"
    if (Point_Size > 0)		# override
    {
	print "\t% NB: The following definition silently overrides the previous one"
	print "\t/TheFontSize PointSize def"
    }
    print "\t/BaselineSkip BaselineSkipFactor TheFontSize mul def"
    print "\tTheFont findfont"
    print "\t\tfindPRIVATE {reencodePRIVATE TheFontPRIVATE exch definefont} if"
    print "\t\tTheFontSize scalefont setfont"
    print "} def"

    print "/NewLine {0 BaselineSkip neg rmoveto} def"

    print "/PosterLines"
    print "["
    for (k = 1; k <= NWord; ++k)
	print "\t(" Word[k] ")"
    print "] def"
    print "/LineCount PosterLines length 1 max def"
    if (Make_Font_Table)
	print_font_table_macros()
    print "end % psposterdict"
    print "%%EndProlog"

    print "%======================================================================="
    print "%%BeginSetup"
    print "/#copies", Copies," def"

    for (k = Embedded_Font_Count; k > 0; --k)
	copyfont(Embedded_Font_Filenames[k],Embedded_Font_Names[k])

    if (font_filename != "")		# download the font
	copyfont(font_filename,Font_Name)

    print "%%EndSetup"

    print "%======================================================================="
    print_page_setup(1)
    if (Make_Font_Table)
    {
	temp_encoding = Encoding
	sub("-PRIVATE","",temp_encoding)
	print "TheFontPRIVATE (" temp_encoding ") MakeFontTable"
    }
    else
    {
	print "% Set font size so that the longest line just fills the TextWidth."
	print "setfontsize"
	print "% Center the text vertically, such that the middle line has its text"
	print "% (rather than baseline distance) centered on a horizontal line"
	print "% passing through the center of the picture."
	print "0 LineCount 0.5 mul BaselineSkip mul BaselineSkip sub moveto"

	print "0 BaselineSkip CapHeight sub 0.5 mul rmoveto"

	print "ForegroundColor setrgbcolor"
	print "% Typeset the poster text."
	print "PosterLines { cshow } forall"
    }
    print_page_trailer()

    if (Display_Fontsize)
    {
	print "%%Page: 2 2"
	print "%%BeginPageSetup"
	print "psposterdict begin"
	print "/tempstr 25 string def"
	print "/Courier findfont 20 scalefont setfont"
	print "/BaselineSkip 20 1.2 mul def"
	print "72 PageHeight 2 div moveto"
	print "(Set font pointsize by) show"
	print "/Courier-BoldOblique findfont 20 scalefont setfont"
	print "( +p ) show"
	print "TheFontSize ScaleFactor div tempstr cvs show"
	print "(bp) show"
	print "showpage"
	print "end % psposterdict"
	print "%%PageTrailer"
    }

    print "%======================================================================="
    print "%%Trailer"
    print "%%EOF"

    exit (0)		# all done (no command-line words are files)
}


function author()
{
    print \
	"Author:\n" \
	"\tNelson H. F. Beebe\n" \
	"\tCenter for Scientific Computing\n" \
	"\tUniversity of Utah\n" \
	"\tDepartment of Mathematics, 322 INSCC\n" \
	"\t155 S 1400 E RM 233\n" \
	"\tSalt Lake City, UT 84112-0090\n" \
	"\tUSA\n" \
	"\tEmail: beebe@math.utah.edu, beebe@acm.org, beebe@ieee.org (Internet)\n" \
	"\tWWW URL: http://www.math.utah.edu/~beebe\n" \
	"\tTelephone: +1 801 581 5254\n" \
	"\tFAX: +1 801 585 1640, +1 801 581 4148" > Stderr
    exit (0)
}


function bigpoint(dimension)
{
    return (72 * inch(dimension))
}


function ceil(x, n)
{
    n = int(x)
    if (n < x) ++n
    return (n)
}


function copyfont(filename,font_name, filter,fontline)
{
    ## NB: .pfb files are binary: gawk can copy them, but nawk cannot.
    ## Even so, before the font can be included in a PostScript file,
    ## it must be converted to ASCII form.  We therefore invoke t1ascii
    ## on-the-fly to handle the conversion, plus gunzip or uncompress,
    ## if needed for decompression.  t1ascii writes a version banner
    ## to stderr, so we redirect it to /dev/null.

    print "%%BeginResource: font " font_name
    if (filename ~ /.pfb$/)	# convert from .pfb to .pfa on-the-fly
	filter = "t1ascii 2>/dev/null " filename
    else if (filename ~ /.pfa.gz$/) # convert from .pfa.gz to .pfa on-the-fly
	filter = "gunzip <" filename
    else if (filename ~ /.pfb.gz$/) # convert from .pfb.gz to .pfa on-the-fly
	filter = "gunzip <" filename " | t1ascii 2>/dev/null "
    else if (filename ~ /.pfa.Z$/) # convert from .pfa.Z to .pfa on-the-fly
	filter = "uncompress <" filename
    else if (filename ~ /.pfb.Z$/) # convert from .pfb.Z to .pfa on-the-fly
	filter = "uncompress <" filename " | t1ascii 2>/dev/null "
    else
	filter = "cat " filename
    print "% file:", filename
    if (filter !~ /^cat /) print "% filter:", filter
    while (filter | getline fontline)
	print fontline
    close (filter)
    print "% font:",font_name
    print "% file:",filename
    print "%%EndResource"
}


function copyright()
{
    print \
	"########################################################################\n" \
	"########################################################################\n" \
	"########################################################################\n" \
	"###                                                                  ###\n" \
	"###  psposter: create simple one-page colored posters in PostScript  ###\n" \
	"###                                                                  ###\n" \
	"###              Copyright (C) 2000 Nelson H. F. Beebe               ###\n" \
	"###                                                                  ###\n" \
	"### This program is covered by the GNU General Public License (GPL), ###\n" \
	"### version 2 or later, available as the file COPYING in the program ###\n" \
	"### source distribution, and on the Internet at                      ###\n" \
	"###                                                                  ###\n" \
	"###               ftp://ftp.gnu.org/gnu/GPL                          ###\n" \
	"###                                                                  ###\n" \
	"###               http://www.gnu.org/copyleft/gpl.html               ###\n" \
	"###                                                                  ###\n" \
	"### This program is free software; you can redistribute it and/or    ###\n" \
	"### modify it under the terms of the GNU General Public License as   ###\n" \
	"### published by the Free Software Foundation; either version 2 of   ###\n" \
	"### the License, or (at your option) any later version.              ###\n" \
	"###                                                                  ###\n" \
	"### This program is distributed in the hope that it will be useful,  ###\n" \
	"### but WITHOUT ANY WARRANTY; without even the implied warranty of   ###\n" \
	"### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    ###\n" \
	"### GNU General Public License for more details.                     ###\n" \
	"###                                                                  ###\n" \
	"### You should have received a copy of the GNU General Public        ###\n" \
	"### License along with this program; if not, write to the Free       ###\n" \
	"### Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   ###\n" \
	"### MA 02111-1307 USA                                                ###\n" \
	"########################################################################\n" \
	"########################################################################\n" \
	"########################################################################" > Stderr
    exit (0)
}


function error(message)
{
    print "FATAL:",message > Stderr
    exit (1)
}


function file_exists(filename)
{
    ## Unfortunately, getline in both nawk and gawk will hang if its
    ## file does not exist, so we need to test for file existence by
    ## invoking a shell command, sigh...

    return (system("test -f " filename) == 0)
}


function find_file(path,filename, fullname,k)
{
    for (k = 1; k in path; ++k)
    {
	fullname = path[k] "/" filename
	if (file_exists(fullname))
	    return (fullname)
    }
    return ("")
}


function find_font(font_name, basename,command,filename,k,mapline)
{
    ## Now see if we have a font that must be downloaded.  If we do not
    ## find it, we silently assume that it is a resident font on the
    ## output PostScript device.  AFMMAPFILE lines look like this:
    ## ZapfChancery-MediumItalic               = ZapChaMI % Optional comment

    if (AFMMAPFILE == "")
	mapline = ""
    else
	mapline = shell_command("grep 2>/dev/null '^" font_name "[ \t]' " AFMMAPFILE)

    k = index(mapline,"=")
    if (k > 0)			# copy font using mapped filename
    {
	basename = substr(mapline,k+1)
	gsub(/%.*$/,"",basename)	# discard inline comment
	gsub(/[ \t]/,"",basename)	# and spaces
    }
    else			# try to copy font using native file name
	basename = font_name

    gsub(/[.]afm$/,"",basename)	# remove optional .afm extension

    filename = find_file(AFM_path, basename ".pfa")
    if (filename == "")
	filename = find_file(AFM_path, basename ".pfb")
    if (filename == "")
	filename = find_file(AFM_path, basename ".pfa.gz")
    if (filename == "")
	filename = find_file(AFM_path, basename ".pfa.Z")
    if (filename == "")
	filename = find_file(AFM_path, basename ".pfb.gz")
    if (filename == "")
	filename = find_file(AFM_path, basename ".pfb.Z")
    return (filename)
}


function floor(x, n)
{
    n = int(x)
    if (n > x) --n
    return (n)
}


function getcolor(value, k,n,rgb,rgbline)
{				# value is e.g., "0.5/0.25/0.9" or "yellow"
    if (value ~ "^#[0-9a-fA-F]+")
    {				# old-style X11 hexadecimal color
	value = substr(value,2)
	n = length(value)
	if ((n % 3) == 0)
	{
	    k = n / 3
	    value = sprintf("%f %f %f", \
			    xtof(substr(value,1 + 0*k,k)), \
			    xtof(substr(value,1 + 1*k,k)), \
			    xtof(substr(value,1 + 2*k,k)))
	}
	else
	{
	    warning("unrecognized hexadecimal color specification [#" value "]: black assumed")
	    value = "0 0 0"
	}
    }
    else if (value ~ "^rgb:[0-9a-fA-F]+/[0-9a-fA-F]+/[0-9a-fA-F]+$")
    {				# new-style X11 hexadecimal color
	split(substr(value,5),rgb,"/")
	value = sprintf("%f %f %f", xtof(rgb[1]), xtof(rgb[2]), xtof(rgb[3]))
    }
    else if (value ~ "^rgbi:[0-9.]+([Ee][-+]?[0-9]+)?/[0-9.]+([Ee][-+]?[0-9]+)?/[0-9.]+([Ee][-+]?[0-9]+)?$")
    {				# new-style X11 fractional intensities
	split(substr(value,6),rgb,"/")
	value = sprintf("%f %f %f", rgb[1], rgb[2], rgb[3])
    }
    else if (value ~ /[A-Za-z]/)
    {				# X11 named color
	gsub(" ","",value)	# remove blanks
	value = tolower(value)	# and reduce to single lettercase
	if (RGBTXTFILE == "")
	    rgbline = "0 0 0"	# default to black
	else
	    rgbline = shell_command("grep 2>/dev/null -i '^[0-9 \t]*" value "[ \t]*$' " RGBTXTFILE)
	split(rgbline,rgb," ")
	value = (rgb[1] / 255.0) " " (rgb[2] / 255.0) " " (rgb[3] / 255.0)
    }
    else			# expect numbered color
    {
	gsub(/[^0-9.]+/," ",value) # eliminate separator characters
	gsub(/^ +/,"",value)
	gsub(/ +$/,"",value)
    }
    return (value)
}


function get_embedded_font_names(font_filename, old_embedded_font_count)
{
    if (font_filename == "")
	return

    ## Adobe Type 1 fonts based on multiple master fonts contain
    ## embedded findfont commands that look something like this:
    ##
    ##	"/AdobeSansMM findfont dup maxlength 18 add dict begin"
    ##
    ## We need to recursively extract the names of such fonts, then
    ## later, we will download them in reverse order, so that each will
    ## be defined before use.

    old_embedded_font_count = Embedded_Font_Count
    while ((getline <font_filename) > 0) # search for an embedded font reference
    {
	if ($0 ~ /^\/[^ ]+ +findfont/)
	{
	    ++Embedded_Font_Count
	    Embedded_Font_Filenames[Embedded_Font_Count] = find_font(substr($1,2))
	    Embedded_Font_Names[Embedded_Font_Count] = substr($1,2)
	}
    }
    close (font_filename)

    if (old_embedded_font_count < Embedded_Font_Count) # then we found an embedded font name
	get_embedded_font_names(Embedded_Font_Filenames[Embedded_Font_Count])
}


function inch(units, value)
{
    value = units + 0					# coerce to number

    if (units ~ "[-+]?[0-9]+[.]?[0-9]*bp")		# big points
	value /= 72.0
    else if (units ~ "[-+]?[0-9]+[.]?[0-9]*in")		# inches
	; # NO-OP
    else if (units ~ "[-+]?[0-9]+[.]?[0-9]*cm")		# centimeters
	value /= 2.54
    else if (units ~ "[-+]?[0-9]+[.]?[0-9]*pt")		# points
	value /= 72.27
    else if (units ~ "[-+]?[0-9]+[.]?[0-9]*cc")		# cicero
	value *= 12.0 * (1238.0 / 1157.0) / 72.27
    else if (units ~ "[-+]?[0-9]+[.]?[0-9]*dd")		# didot points
	value *= (1238.0 / 1157.0) / 72.27
    else if (units ~ "[-+]?[0-9]+[.]?[0-9]*mm")		# millimeters
	value /= 25.4
    else if (units ~ "[-+]?[0-9]+[.]?[0-9]*pc")		# pica
	value *= 12.0 / 72.27
    else if (units ~ "[-+]?[0-9]+[.]?[0-9]*sp")		# scaled points
	value /= (65536.0 * 72.27)
    else
	error("unrecognized dimension " units)
    return (value)
}


function initialize_paper_table()
{
    ## name                        width    height    left    right   bottom  top

    ## ANSI (note that B folded in half becomes A, etc.) (portrait)

    Paper["A"]			= "8.5in    11in     1in     1in     1in    1in"
    Paper["B"]			= "11in     17in     1in     1in     1in    1in"
    Paper["C"]			= "17in     22in     1in     1in     1in    1in"
    Paper["D"]			= "22in     34in     1in     1in     1in    1in"
    Paper["E"]			= "34in     44in     1in     1in     1in    1in"

    ## Common US designations (portrait)

    Paper["Computer-1411"]	= "14in     11in     1in     1in     1in    1in"
    Paper["Legal"]		= "8.5in    13in     1in     1in     1in    1in"
    Paper["Letter"]		= "8.5in    11in     1in     1in     1in    1in"
    Paper["US-Legal"]		= "8.5in    14in     1in     1in     1in    1in"

    ## Common US designations (landscape)

    Paper["A-L"]		= "11in     8.5in    1in     1in     1in    1in"
    Paper["Computer-1411-L"]	= "11in     14in     1in     1in     1in    1in"
    Paper["Legal-L"]		= "13in     8.5in    1in     1in     1in    1in"
    Paper["Letter-L"]		= "11in     8.5in    1in     1in     1in    1in"
    Paper["US-Legal-L"]		= "14in     8.5in    1in     1in     1in    1in"

    ## Additional paper and envelope sizes taken from <<Hewlett-Packard
    ## LaserJet Family Paper Specification Guide>> (1992).

    Paper["COM10"]		= "4.1in    9.5in    1in     1in     1in    1in"
    Paper["DL"]			= "110mm    220mm    25mm    25mm    25mm   25mm"
    Paper["Executive"]		= "7.25in   10.5in   1in     1in     1in    1in"
    Paper["Monarch"]		= "3.9in    7.5in    1in     1in     1in    1in"

    ## Miscellaneous and Imagen Ultrascript paper sizes

    Paper["A4Small"]		= "210mm    297mm    25mm    25mm    25mm   25mm"
    Paper["Ledger"]		= "11in     17in     1in     1in     1in    1in"
    Paper["LetterSmall"]	= "8.5in    11in     1in     1in     1in    1in"
    Paper["Note"]		= "8.5in    11in     1in     1in     1in    1in"

    ## ISO standard paper A-series (portrait).   Note that Xn folded in half
    ## becomes Xn+1 (X = A, B, C)

    Paper["A0"]			= "841mm    1189mm   25mm    25mm    25mm   25mm"
    Paper["A1"]			= "594mm    841mm    25mm    25mm    25mm   25mm"
    Paper["A2"]			= "420mm    594mm    25mm    25mm    25mm   25mm"
    Paper["A3"]			= "297mm    420mm    25mm    25mm    25mm   25mm"
    Paper["A4"]			= "210mm    297mm    25mm    25mm    25mm   25mm"
    Paper["A5"]			= "148mm    210mm    25mm    25mm    25mm   25mm"
    Paper["A6"]			= "105mm    148mm    25mm    25mm    25mm   25mm"
    Paper["A7"]			= "74mm     105mm    25mm    25mm    25mm   25mm"
    Paper["A8"]			= "52mm     74mm     25mm    25mm    25mm   25mm"
    Paper["A9"]			= "37mm     52mm     25mm    25mm    25mm   25mm"
    Paper["A10"]		= "26mm     37mm     25mm    25mm    25mm   25mm"

    ## ISO standard paper A-series (landscape).  Note that the final L
    ## is NOT prefixed by a hyphen.

    Paper["A0L"]		= "1189mm   841mm    25mm    25mm    25mm   25mm"
    Paper["A1L"]		= "841mm    594mm    25mm    25mm    25mm   25mm"
    Paper["A2L"]		= "594mm    420mm    25mm    25mm    25mm   25mm"
    Paper["A3L"]		= "420mm    297mm    25mm    25mm    25mm   25mm"
    Paper["A4L"]		= "297mm    210mm    25mm    25mm    25mm   25mm"
    Paper["A5L"]		= "210mm    148mm    25mm    25mm    25mm   25mm"
    Paper["A6L"]		= "148mm    105mm    25mm    25mm    25mm   25mm"
    Paper["A7L"]		= "105mm    74mm     25mm    25mm    25mm   25mm"
    Paper["A8L"]		= "74mm     52mm     25mm    25mm    25mm   25mm"
    Paper["A9L"]		= "52mm     37mm     25mm    25mm    25mm   25mm"
    Paper["A10L"]		= "37mm     26mm     25mm    25mm    25mm   25mm"

    ## ISO intermediate paper B-series (portrait)

    Paper["B0"]			= "1000mm   1414mm   25mm    25mm    25mm   25mm"
    Paper["B1"]			= "707mm    1000mm   25mm    25mm    25mm   25mm"
    Paper["B2"]			= "500mm    707mm    25mm    25mm    25mm   25mm"
    Paper["B3"]			= "353mm    500mm    25mm    25mm    25mm   25mm"
    Paper["B4"]			= "250mm    353mm    25mm    25mm    25mm   25mm"
    Paper["B5"]			= "176mm    250mm    25mm    25mm    25mm   25mm"
    Paper["B6"]			= "125mm    176mm    25mm    25mm    25mm   25mm"

    ## ISO intermediate paper B-series (landscape)

    Paper["B0L"]		= "1414mm   1000mm   25mm    25mm    25mm   25mm"
    Paper["B1L"]		= "1000mm   707mm    25mm    25mm    25mm   25mm"
    Paper["B2L"]		= "707mm    500mm    25mm    25mm    25mm   25mm"
    Paper["B3L"]		= "500mm    353mm    25mm    25mm    25mm   25mm"
    Paper["B4L"]		= "353mm    250mm    25mm    25mm    25mm   25mm"
    Paper["B5L"]		= "250mm    176mm    25mm    25mm    25mm   25mm"
    Paper["B6L"]		= "176mm    125mm    25mm    25mm    25mm   25mm"

    ## ISO envelopes for A-series

    Paper["C0"]			= "1294mm   916mm    25mm    25mm    25mm   25mm"
    Paper["C1"]			= "916mm    647mm    25mm    25mm    25mm   25mm"
    Paper["C2"]			= "647mm    458mm    25mm    25mm    25mm   25mm"
    Paper["C3"]			= "458mm    323mm    25mm    25mm    25mm   25mm"
    Paper["C4"]			= "323mm    229mm    25mm    25mm    25mm   25mm"
    Paper["C5"]			= "229mm    161mm    25mm    25mm    25mm   25mm"
    Paper["C6"]			= "161mm    114mm    25mm    25mm    25mm   25mm"

    ## British paper sizes (some are also used in the USA)

    Paper["Octavo"]		= "5in      8in      1in     1in     1in    1in"
    Paper["Sixmo"]		= "6.5in    8in      1in     1in     1in    1in"
    Paper["Quarto"]		= "8in      10in     1in     1in     1in    1in"
    Paper["Foolscap"]		= "8.5in    13in     1in     1in     1in    1in"
    Paper["Government-legal"]	= "8.5in    13in     1in     1in     1in    1in"
    Paper["Folio"]		= "8.3in    13in     1in     1in     1in    1in"
}


function parse_table_type(s, parts,t)
{
    t = s
    gsub(/[^0-9A-Za-z]/," ",t)
    split(t,parts," ")
    Table_Columns = 0 + parts[1]
    Label_Base = (parts[2] ~ /^[oO]$/) ? 8 : ((parts[2] ~ /^[hH]$/) ? 16 : 0)
    if (((Table_Columns != 8) && (Table_Columns != 16)) || \
	((Label_Base != 8) && (Label_Base != 16)))
    {
	warning("Unrecognized +table value `" s \
	    "': expected one of 8/o 16/o 8/h 16/h.  Defaulting to 8/o.")
	parse_table_type("8/o")
    }
    Make_Font_Table = s
}


function print_default_encoding()
{
    Encoding = "StandardEncoding"
}


function print_font_table_macros()
{
    print "/Black { 0 0 0 } def"
    print "/LabelBase " Label_Base " def"
    print "/TableColumns " Table_Columns " def"
    print "/LabelChars 3 def               % 0x3_ or \\12_; 3 looks better than 4"
    print "/RowChars TableColumns def"
    print "/TheRow RowChars string def     % 8 or 16 characters"
    print "/TitleLines TableColumns 8 eq LabelBase 16 eq and { 4 } { 3 } ifelse def"
    print "/TableLines 256 TableColumns div TitleLines add def % 16 or 32 char rows and 2 or 3 title rows"
    print "PointSize 0 eq"
    print "\t{"
    print "\t\t/dx"
    print "\t\t\tTextWidth RowChars LabelChars add 2 mul div"
    print "\t\t\tTextHeight TableLines BaselineSkipFactor mul div"
    print "\t\t\tmin"
    print "\t\t\tcvi"
    print "\t\tdef"
    print "\t}"
    print "\t{"
    print "\t\t/dx PointSize def"
    print "\t}"
    print "ifelse"

    ## [11-Jan-2000] TheFontSize must be an integer, since it is
    ## converted to a 4-character string by cvs in TitleString below
    print "/TheFontSize dx ceiling def"

    print "/BaselineSkip BaselineSkipFactor TheFontSize mul def"
    print "/LabelWidth dx LabelChars mul def"
    print "/TableHeight BaselineSkip TableLines mul def"
    print "/TableWidth dx 2 mul RowChars mul dx add LabelWidth add def"
    print "/TitleBoxHeight BaselineSkip TitleLines mul def"
    print ""
    print "/KernShow"
    print "{ % Usage: (string) KernShow % show with characters of width 2*dx"
    print "\tSavePosition"
    print "\t{                           % stack: c(n) c(n+1)"
    print "\t\tpop pop"
    print "\t\t/x x dx 2 mul add def"
    print "\t\tx y moveto"
    print "\t}"
    print "\texch kshow"
    print "} def"
    print "/MakeFontTable"
    print "{ % Usage: /fontname (encoding-name) MakeFontTable"
    print "  % (0,0) is at page center"
    print "\t/MakeFontEncodingName exch def"
    print "\tfindfont TheFontSize scalefont setfont"
    print "\tTableWidth -2 div TableHeight 2 div moveto"
    print "\tShowTableTitle"
    print "\tgsave"
    print "\t\tSetLabelFont"
    print "\t\tSetLabelColor"
    print "\t\tLabelWidth dx add 0 rmoveto"
    print "\t\tTableColumns 16 eq"
    print "\t\t\t{"
    print "\t\t\t\tLabelBase 16 eq"
    print "\t\t\t\t\t{(0123456789abcdef)}"
    print "\t\t\t\t\t{(0123456701234567)}"
    print "\t\t\t\tifelse"
    print "\t\t\t}"
    print "\t\t\t{"
    print "\t\t\t\t(01234567)"
    print "\t\t\t}"
    print "\t\tifelse"
    print "\t\tKernShow"
    print "\tgrestore"
    print "\tTableColumns 8 eq LabelBase 16 eq and"
    print "\t\t{"
    print "\t\t\tNewLine"
    print "\t\t\tgsave"
    print "\t\t\t\tSetLabelFont"
    print "\t\t\t\tSetLabelColor"
    print "\t\t\t\tLabelWidth dx add 0 rmoveto"
    print "\t\t\t\t(89abcdef) KernShow"
    print "\t\t\tgrestore"
    print "\t\t}"
    print "\tif"
    print "\tSavePosition"
    print "\t1 setlinewidth"
    print "\tSetLabelColor"
    print "\tx y TitleBoxHeight add TableWidth ShowHorizontalRule"
    print "\tx y TableWidth ShowHorizontalRule"
    print "\tx y TitleBoxHeight add TableHeight ShowVerticalRule"
    print "\tx LabelWidth add y TitleBoxHeight add TableHeight ShowVerticalRule"
    print "\tx TableWidth add y TitleBoxHeight add TableHeight ShowVerticalRule"
    print "\tTableColumns 16 eq"
    print "\t\t{"
    print "\t\t\tx LabelWidth add TableWidth LabelWidth sub 2 div add"
    print "\t\t\ty TableHeight TitleBoxHeight sub ShowVerticalRule"
    print "\t\t}"
    print "\tif"
    print "\tx y moveto"
    print "\tNewLine"
    print "\tForegroundColor setrgbcolor"
    print "\tShowCharacterTable"
    print "\tSetLabelColor"
    print "\tReverseNewline"
    print "\tx y TableWidth ShowHorizontalRule"
    print "} def"
    print "/NewLine {0 BaselineSkip neg rmoveto SavePosition} def"
    print "/ReverseNewline {0 BaselineSkip rmoveto SavePosition} def"
    print "/RuleVerticalOffset BaselineSkip -0.2 mul def"
    print "/SavePosition { currentpoint /y exch def /x exch def } def"
    print "/SetLabelColor { Black setrgbcolor } def"
    print "/SetLabelFont"
    print "{"
    print "\t/Times-BoldItalic findfont TheFontSize scalefont setfont"
    print "} def"
    print "/ShowCharacterTable"
    print "{"
    print "\t0 TableColumns 255"
    print "\t{"
    print "\t\t/n exch def"
    print "\t\tn 8#100 mod 8#070 eq"
    print "\t\t{"
    print "\t\t\tgsave"
    print "\t\t\t    SetLabelColor"
    print "\t\t\t    x y TableWidth ShowHorizontalRule"
    print "\t\t\tgrestore"
    print "\t\t} if"
    print "\t\tShowRowLabel"
    print "\t\tgsave"
    print "\t\t\tLabelWidth dx add 0 rmoveto"
    print "\t\t\tShowRow"
    print "\t\tgrestore"
    print "\t\tNewLine"
    print "\t} for"
    print "} def"
    print "/ShowHorizontalRule"
    print "{   % Usage: x y len ShowHorizontalRule"
    print "\tgsave"
    print "\t\t/r exch def"
    print "\t\tnewpath"
    print "\t\t\tmoveto"
    print "\t\t\t0 RuleVerticalOffset rmoveto"
    print "\t\t\tr 0 rlineto"
    print "\t\tstroke"
    print "\tgrestore"
    print "} def"
    print "/ShowRow"
    print "{   % Usage: ShowRow"
    print "\tgsave"
    print "\t\t0 1 TableColumns 1 sub"
    print "\t\t{"
    print "\t\t\t/k exch def"
    print "\t\t\tTheRow  k  n k add  put % char"
    print "\t\t} for   % create character row"
    print "\t\tSavePosition"
    print "\t\tTheRow KernShow"
    print "\tgrestore"
    print "} def"
    print "/ShowRowLabel"
    print "{   % Usage: ShowRowLabel"
    print "\tcurrentpoint"
    print "\tgsave"
    print "\t\tSetLabelColor"
    print "\t\tSetLabelFont"
    print "\t\tLabelBase 16 eq"
    print "\t\t\t{"
    print "\t\t\t\t/TempStr 5 string def"
    print "\t\t\t\tTempStr 0 (   ) putinterval"
    print "\t\t\t\tn 16 idiv 8#10000 add 16 TempStr cvrs pop"
    print "\t\t\t\t/k 0 def"
    print "\t\t\t\tTempStr"
    print "\t\t\t\t{ % convert to lowercase"
    print "\t\t\t\t\tdup dup 8#100 ge exch 8#107 le and { 32 add } if"
    print "\t\t\t\t\tTempStr exch k exch put"
    print "\t\t\t\t\t/k k 1 add def"
    print "\t\t\t\t} forall"
    print "\t\t\t\tTempStr 0 (\\\\0x) putinterval"
    print "\t\t\t\tTempStr 4 (_) putinterval"
    print "\t\t\t}"
    print "\t\t\t{"
    print "\t\t\t\t/TempStr 4 string def"
    print "\t\t\t\tTempStr 0 (   ) putinterval"
    print "\t\t\t\tn 8 idiv 8#100 add 8 TempStr cvrs pop"
    print "\t\t\t\tTempStr 0 (\\\\) putinterval"
    print "\t\t\t\tTempStr 3 (_) putinterval"
    print "\t\t\t}"
    print "\t\tifelse"
    print "\t\tLabelWidth 2 div 0 rmoveto"
    print "\t\tTempStr cshow"
    print "\tgrestore"
    print "\tmoveto"
    print "\tSavePosition"
    print "} def"
    print "/ShowTableTitle"
    print "{   % Usage: ShowTableTitle"
    print "\tNewLine"
    print "\tgsave"
    print "\t\tSetLabelColor"
    print "\t\tSetLabelFont"
    print "\t\tLabelWidth 0 rmoveto"
    print "\t\tTableWidth LabelWidth sub 2 div 0 rmoveto"
    print "\t\t% Create centered title, e.g., 23bp FontName"
    print "\t\t/TitleString TheFont length 5 add string def"
    print "\t\tTitleString 0 TheFontSize 4 string cvs putinterval"
    print "\t\tTitleString 2 (bp ) putinterval"
    print "\t\tTitleString 5 TheFont dup length string cvs putinterval"
    print "\t\tTitleString cshow"
    print "\t\tMakeFontEncodingName cshow"
    print "\tgrestore"
    print "\tNewLine"
    print "\tNewLine"
    print "} def"
    print "/ShowVerticalRule"
    print "{   % Usage: x y len ShowVerticalRule"
    print "\tgsave"
    print "\t\t/r exch def"
    print "\t\tnewpath"
    print "\t\t\tmoveto"
    print "\t\t\t0 RuleVerticalOffset rmoveto"
    print "\t\t\t0 r neg rlineto"
    print "\t\tstroke"
    print "\tgrestore"
    print "} def"
}


function print_page_setup(page_number)
{
    print "%%Page:", page_number, page_number
    print "%%BeginPageSetup"
    print "psposterdict begin"
    print "XOffset YOffset translate"
    print "ScaleFactor dup scale"
    print "ROTATE_" (Angle + 0)
    print "end % psposterdict"
    print "%%EndPageSetup"
    print ""
    print "psposterdict begin"
    print "% Draw a frame box and fill it with the background color, and a border color."
    print "BorderWidth setlinewidth"
    print "newpath"
    print "MarginWidth BorderWidth 0.5 mul add dup moveto"
    print "TextWidth BorderMargin 2 mul add BorderWidth add 0 rlineto"
    print "0 TextHeight BorderMargin 2 mul add BorderWidth add rlineto"
    print "TextWidth BorderMargin 2 mul add BorderWidth add neg 0 rlineto"
    print "closepath"
    print "BackgroundColor setrgbcolor gsave fill grestore"
    print "BorderColor setrgbcolor stroke"

    print ""
    print "0 0 0 setrgbcolor % revert to black"

    print ""
    print "% Set the clipping path to the text region"
    print "0 setlinewidth"
    print "newpath"
    print "MarginWidth BorderWidth add BorderMargin add dup moveto"
    print "TextWidth 0 rlineto"
    print "0 TextHeight rlineto"
    print "TextWidth neg 0 rlineto"
    print "closepath"
    print "clip"

    print ""
    print "% Move origin to text box center, since x = 0 is a convenient"
    print "% value for centering text."
    print "LandscapeMode"
    print "\t{ Height 0.5 mul Width 0.5 mul translate }"
    print "\t{ Width 0.5 mul Height 0.5 mul translate }"
    print "ifelse"
}


function print_page_trailer()
{
    print "end % psposterdict"
    print "showpage"
    print "%%PageTrailer"
}


function print_private_encoding( filename,k,line,n,name,parts)
{
    ## Find the private encoding file corresponding to the global name Encoding,
    ## and output it to stdout.
    if (file_exists(Encoding))
	filename = Encoding
    else if (file_exists(Encoding ".enc"))
	filename = Encoding ".enc"
    else
	filename = find_file(AFM_path,Encoding ".enc")
    if (filename == "")
    {
	warning("cannot find encoding [" Encoding "]: defaulting to Adobe Standard Encoding")
	print_default_encoding()
    }
    else
    {
	print "%%BeginResource: Encoding %", filename
	name = ""
	while ((getline line < filename) > 0)
	{
	    if (line ~ "^[ \t]*%")	# discard whole-line comments
		continue
	    else if (line ~ "^[ \t]*$")	# discard empty lines
		continue
	    else if ((name == "") && (line ~ "^[ \t]*/"))
	    {
		## Expect "/Latin-1 ...": change to "/Latin-1-PRIVATE ..."
		n = split(line,parts,"[ \t]+")
		name = substr(parts[1],2)
		Encoding = name "-PRIVATE"
		line = "/" Encoding
		for (k = 2; k <= n; ++k)
		    line = (line " " parts[k])
	    }
	    print line
	}
	print "%%EndResource"
	close(filename)
    }
}


function set_paper_dimensions(name, n,swap,value)
{
    if (name in Paper)
    {
	n = split(Paper[name],value," ")
	if (n == 6)
	{
	    Paper_Name   = name
	    Paper_Width  = bigpoint(value[1])
	    Paper_Height = bigpoint(value[2])
	    Paper_Left   = bigpoint(value[3])
	    Paper_Right  = bigpoint(value[4])
	    Paper_Bottom = bigpoint(value[5])
	    Paper_Top    = bigpoint(value[6])
	}
	else
	    error("internal error: corrupt paper dimensions for paper type " name)
    }
    else
    {	# expect something like 8.5inx11in
	n = split(name,value,"x")
	if (n == 2)
	{
	    Paper_Name   = name
	    Paper_Width  = bigpoint(value[1])
	    Paper_Height = bigpoint(value[2])
	    Paper_Left   = bigpoint("1in")
	    Paper_Right  = bigpoint("1in")
	    Paper_Bottom = bigpoint("1in")
	    Paper_Top    = bigpoint("1in")
	}
	else
	    error("unrecognized paper type " name)
    }
    ## [11-Jan-2000] Comment out.  In psposter, landscape refers
    ## to text orientation, not paper dimensions.  This behavior
    ## is documented in the manual pages.
    ##	if (Paper_Width > Paper_Height)
    ##	    Landscape = 1
}


function set_text_dimensions()
{
    Text_Width = Width - 2*Margin_Width - 2*Border_Width - 2*Border_Margin
    Text_Height = Height - 2*Margin_Width - 2*Border_Width - 2*Border_Margin
}


function shell_command(command, s)
{
    ## Run a shell command and return its first output line, after
    ## closing the file.
    command | getline s
    close(command)
    return (s)
}


function source()
{
    print \
	"Internet source distribution:\n" \
	"\tURL: ftp://ftp.math.utah.edu/pub/misc/psposter-" VERSION_NUMBER ".jar\n" \
	"\tURL: ftp://ftp.math.utah.edu/pub/misc/psposter-" VERSION_NUMBER ".tar.gz\n" \
	"\tURL: ftp://ftp.math.utah.edu/pub/misc/psposter-" VERSION_NUMBER ".zip\n" \
	"\tURL: ftp://ftp.math.utah.edu/pub/misc/psposter-" VERSION_NUMBER ".zoo\n" \
	"\tURL: http://www.math.utah.edu/pub/misc/psposter-" VERSION_NUMBER ".jar\n" \
	"\tURL: http://www.math.utah.edu/pub/misc/psposter-" VERSION_NUMBER ".tar.gz\n" \
	"\tURL: http://www.math.utah.edu/pub/misc/psposter-" VERSION_NUMBER ".zip\n" \
	"\tURL: http://www.math.utah.edu/pub/misc/psposter-" VERSION_NUMBER ".zoo" > Stderr
    exit (0)
}


function usage()
{
    print \
	"Usage: psposter [+option value] ... word1 word2 ...\n" \
	"\n" \
	"Recognized options are:\n" \
	"\t+author          display program author and quit\n" \
	"\t+bd color        set border color\n" \
	"\t+bg color        set background color\n" \
	"\t+bm dimension    set border margin\n" \
	"\t+bs factor       set baselineskip to factor times font pointsize\n" \
	"\t+bw dimension    set border width\n" \
	"\t+c nnn           number of copies to print\n" \
	"\t+copyright       display program copyright and quit\n" \
	"\t+d               display font pointsize on second page\n" \
	"\t+e encoding      specify a character-set encoding for the font\n" \
	"\t+fg color        set foreground color\n" \
	"\t+fn fontname     select PostScript font\n" \
	"\t+h dimension     poster height\n" \
	"\t+help            display this message and quit\n" \
	"\t+l               landscape (instead of portrait) orientation\n" \
	"\t+mw dimension    set margin width\n" \
	"\t+p dimension     set font pointsize\n" \
	"\t+paper form      set paper size\n" \
	"\t+r angle         set text rotation angle in degrees (only multiples of 90)\n" \
	"\t+s factor        set poster scale factor\n" \
	"\t+source          show software source archive location\n" \
	"\t+table {8/o | 8/h | 16/o | 16/h}   show font character table\n" \
	"\t+version         display program version and quit\n" \
	"\t+w dimension     set poster width\n" \
	"\t+x dimension     set x offset\n" \
	"\t+y dimension     set y offset\n" \
	"\t?                display this message and quit\n" \
	"\t++               no more options: remaining words are for poster text\n" \
	"\n" \
	"Any command-line arguments that are not recognized as options will be\n" \
	"typeset on the poster, one argument per line.  If no poster text words\n" \
	"are supplied on the command line, then poster lines are read from\n" \
	"standard input.\n" \
	"\n" \
	"Color names may be any named or hexadecimal or `rgb:' or `rgbi:' X\n" \
	"Window System color, or red, green, and blue values in 0..1 separated by\n" \
	"characters other than digits or period.  E.g., green, #0f0, #00ff00,\n" \
	"#000fff00, #0000ffff0000, rgb:0/f/0, rgb:0/ff/0, rgb:0/fff/0,\n" \
	"rgb:0/ffff/0, rgbi:0/1/0, and 0/1/0 all represent green, and gray50,\n" \
	"grey50, #888, #808080, #800800800, #800080008000, rgb:8/8/8,\n" \
	"rgb:80/80/80, rgb:800/800/800, rgb:8000/8000/8000, rgbi:0.5/0.5/0.5, and\n" \
	"0.5:0.5:0.5 all represent 50% grey.\n" \
	"\n" \
	"Dimensions consist of a (possibly signed) number followed by a TeX\n" \
	"dimension unit: bp (big points), cc (ciceros), cm (centimeters), dd\n" \
	"(didot points), in (inches), mm (millimeters), pc (picas), pt (points),\n" \
	"or sp (scaled points).  E.g. 72bp, 2.54cm, 67.542dd, 1in, 25.4mm,\n" \
	"6.023pc, 72.27pt, and 4736286.72sp all specify a dimension of one inch.\n" \
	"\n" \
	"Paper names may be either standard ones like A, A4, Foolscap, etc., or\n" \
	"width and height dimensions separated by an x, so that 210mmx297mm and\n" \
	"A4 are equivalent, and 8.5inx11in and A are too." > Stderr

    exit (0)
}


function version()
{
    print "psposter", VERSION > Stderr
    exit (0)
}


function warning(message)
{
    print message >Stderr
}


function xtof(x, n)
{
    ## Convert an unsigned hexadecimal string to a scaled
    ## floating-point value in 0..1, and return the result.  The scale
    ## factor is determined by the number of hexadecimal digits: 15.0
    ## for 1 digit, 255.0 for 2, 4095.0 for 3, and 65535 for 4 or
    ## more.  Any digits beyond the fourth are ignored.
    x = substr(x,1,4)		# ignore trailing digits
    n = length(x)
    if (n == 0)
	return (0.0)
    else if (n == 1)
	return (xtoi(x) / 15.0)
    else if (n == 2)
	return (xtoi(x) / 255.0)
    else if (n == 3)
	return (xtoi(x) / 4095.0)
    else
	return (xtoi(x) / 65535.0)
}


function xtoi(x, h,n)
{
    ## Convert an unsigned hexadecimal string to a decimal integer and
    ## return the result.
    x = tolower(x)
    n = 0
    while (x != "")
    {
	h = index("0123456789abcdef",substr(x,1,1)) - 1
	if (h < 0)
	    return (n)
	n = 16*n + h
	x = substr(x,2)
    }
    return (n)
}
